#include <stdio.h>
#include "bcdefn.h"
#include "pcilib.h"

int pci_bios_present (BC_BYTE *hardware_mechanism, BC_WORD *interface_level_version, BC_BYTE *last_bus_number)
{
   BC_DWORD edx_reg = 0;
   BC_BYTE ah_reg = 0;
   BC_BYTE al_reg = 0;
   BC_WORD bx_reg = 0;
   BC_BYTE cl_reg = 0;

   __asm {
      /* preserve registers */
      _emit 0x66
      push ax
      _emit 0x66
      push bx
      _emit 0x66
      push cx
      _emit 0x66
      push dx

      mov ah, PCI_FUNCTION_ID
      mov al, PCI_BIOS_RESIDENT

      int 1Ah

      _emit 0x66
      mov WORD PTR edx_reg, DX
      mov ah_reg,  AH
      mov al_reg,  AL
      mov bx_reg,  BX
      mov cl_reg,  CL

      /* restore registers */
      _emit 0x66
      pop dx
      _emit 0x66
      pop cx
      _emit 0x66
      pop bx
      _emit 0x66
      pop ax

   }

   if ( (edx_reg & 0xff)	 == 'P' &&
      ( (edx_reg >> 8) & 0xff )  == 'C' &&
      ( (edx_reg >> 16) & 0xff ) == 'I' &&
      ( (edx_reg >> 24) & 0xff ) == ' ' &&
	ah_reg == 0 )
   {
      if(hardware_mechanism)		*hardware_mechanism = al_reg;
      if(interface_level_version)	*interface_level_version = bx_reg;
      if(last_bus_number)			*last_bus_number = cl_reg;

      return (TRUE);
   }
   else
      return (FALSE);
}

int pci_find_device(BC_WORD device_id, BC_WORD vendor_id, BC_WORD index, BC_BYTE *bus_number, BC_BYTE *dev_func_number)
{
   BC_BYTE bh_reg = 0;
   BC_BYTE bl_reg = 0;
   BC_BYTE ah_reg = 0;

   __asm {
      /* preserve registers */
      _emit 0x66
      push ax
      _emit 0x66
      push bx
      _emit 0x66
      push cx
      _emit 0x66
      push dx


      mov ah, PCI_FUNCTION_ID
      mov al, PCI_FIND_PCI_DEVICE
      mov cx, device_id
      mov dx, vendor_id
      mov si, index

      int 1Ah

      mov bh_reg,  BH
      mov bl_reg,  BL
      mov ah_reg,  AH

      /* restore registers */
      _emit 0x66
      pop dx
      _emit 0x66
      pop cx
      _emit 0x66
      pop bx
      _emit 0x66
      pop ax
   }

   if ( ah_reg != PCI_DEVICE_NOT_FOUND )
   {
      *bus_number = bh_reg;
      *dev_func_number = bl_reg;
      return(TRUE);
   }
   else
      return(FALSE);
}

BC_DWORD pci_read_cs32(BC_BYTE bus_number, BC_BYTE dev_func_number, BC_WORD regnum)
{

   BC_DWORD ecx_reg = 0;
   BC_BYTE ah_reg = 0;

   __asm {
      /* preserve registers */
      _emit 0x66
      push ax
      _emit 0x66
      push bx
      _emit 0x66
      push cx
      _emit 0x66
      push dx

      mov ah, PCI_FUNCTION_ID
      mov al, PCI_READ_CS_DWORD
      mov bh, bus_number
      mov bl, dev_func_number
      mov di, regnum

      int 1Ah

      _emit 0x66
      mov WORD PTR ecx_reg,  CX
      mov ah_reg,  AH

      /* restore registers */
      _emit 0x66
      pop dx
      _emit 0x66
      pop cx
      _emit 0x66
      pop bx
      _emit 0x66
      pop ax
   }

   if ( ah_reg == PCI_SUCCESSFUL )
   {
      return ( ecx_reg );
   }
   else
      return ( (BC_DWORD)-1 );
}

BC_BYTE pci_read_cs8(BC_BYTE bus_number, BC_BYTE dev_func_number, BC_WORD regnum)
{

   BC_BYTE cl_reg = 0;
   BC_BYTE ah_reg = 0;

   __asm {
      /* preserve registers */
      _emit 0x66
      push ax
      _emit 0x66
      push bx
      _emit 0x66
      push cx
      _emit 0x66
      push dx

      mov ah, PCI_FUNCTION_ID
      mov al, PCI_READ_CS_BYTE
      mov bh, bus_number
      mov bl, dev_func_number
      mov di, regnum

      int 1Ah

      _emit 0x66
      mov  cl_reg,  CL
      mov ah_reg,  AH

      /* restore registers */
      _emit 0x66
      pop dx
      _emit 0x66
      pop cx
      _emit 0x66
      pop bx
      _emit 0x66
      pop ax
   }

   if ( ah_reg == PCI_SUCCESSFUL )
   {
      return ( cl_reg );
   }
   else
      return ( (BC_BYTE)-1 );
}

int pci_write_cs32(BC_BYTE bus_number, BC_BYTE dev_func_number, BC_WORD regnum, BC_DWORD data_value)
{
   BC_BYTE ah_reg = 0;

   __asm {
      /* preserve registers */
      _emit 0x66
      push ax
      _emit 0x66
      push bx
      _emit 0x66
      push cx
      _emit 0x66
      push dx

      mov ah, PCI_FUNCTION_ID
      mov al, PCI_WRITE_CS_DWORD
      mov bh, bus_number
      mov bl, dev_func_number
      mov di, regnum
      _emit 0x66
      mov cx, WORD PTR data_value

      int 1Ah
    
      mov ah_reg,  AH

      /* restore registers */
      _emit 0x66
      pop dx
      _emit 0x66
      pop cx
      _emit 0x66
      pop bx
      _emit 0x66
      pop ax
   }

   if ( ah_reg == PCI_SUCCESSFUL )
   {
      return ( TRUE );
   }
   else
      return ( FALSE );
      
}

BC_DWORD in32(BC_DWORD address)
{
   BC_WORD addr;
   BC_DWORD eax_reg;

   /* note address must be 16 bit for IN instruction */
   addr = (BC_WORD) address;

   _asm {
        push    dx
        _emit      0x66
        push    ax      /* push eax */

        mov     dx, addr
        _emit      0x66
        in      ax, dx    

        _emit      0x66
        mov     WORD PTR eax_reg, AX 

        _emit      0x66
        pop     ax
        pop     dx
   }

   return ( eax_reg );
}

int out32(BC_DWORD address, BC_DWORD data)
{
   BC_WORD addr;

   /* note address must be 16 bit for OUT instruction */
   addr = (BC_WORD) address;

   _asm {
      push      dx
      _emit 0x66
      push      ax

      mov       dx, addr
      _emit        0x66
      mov       ax, WORD PTR data

      _emit        0x66
      out       dx, ax    

      _emit 0x66
      pop       ax
      pop       dx
   }

   return( TRUE );
}

